define(function(require) {
    $('html, body').height('auto');
    
    var apiUrl = require('../apiConfig');

    var curStep;
    var skInfo;
    var orderStatus=3;

    var SkInfo = Backbone.Model.extend({
        url: apiUrl.addr + '?r=store.tinyshop.login',
        sync: function() {
            var self = this;
            $.ajax({
                url: this.url,
                type: 'post',
                data: {
                    mobile_no: context.user.mobileNo,
                    username: context.user.realName,
                    token: context.user.token
                },
                success: function(data, textStatus, jqXHR){
                    if (data.success) {
                        sessionStorage.tinyshop_id = data.tinyshop.id;
                        skInfo = data.tinyshop;
                        myWeidianView.renderShop(data.tinyshop);
                        myWeidianView.collection.fetch(); //回调成功后，获取商品列表
                    } 
                    else {
                        $('#loading').fadeOut(200, function() {
                            alert(data.msg, function() {}, '提示', '确定');
                        });
                    }
	            },
	            error: function( jqXHR, textStatus, errorThrown ){
	            	//self.sync();
	            	$('#loading').fadeOut(200, function(e) {
                        alert('请检查当前网络', function() {}, '提示', '确定');
                    });
	            }
            });
        }
    });

    var GoodList = Backbone.Collection.extend({
        url: apiUrl.addr + '?r=store.product.list',
        sync: function() {
            var self = this;
            $.ajax({
                url: this.url,
                type: 'get',
                data: {
                    mobile_no: context.user.mobileNo
                },
                success: function(data) {
                    if (data.success) {
                        self.reset(data.product_list);
                        //显示页面
                        require.async('../nav', function(nav) {
                            nav.init(curStep);
                        });
                        $('#loading').fadeOut(200);
                        $('.goods_no').text(data.product_list.length);
                    } else {
                        self.goBackUrl();
                        $('#loading').fadeOut(200, function() {
                            alert(data.msg, function() {}, '提示', '确定');
                        });
                    }
                },
                error: function() {
                    self.goBackUrl();
                    $('#loading').fadeOut(200, function(e) {
                        alert('请检查当前网络', function() {}, '提示', '确定');
                    });
                }
            });
        },
        goBackUrl: function() {
            switch (curStep) {
                case '4':
                    app_router.navigate('!shopEidt', {trigger: false});
                    break;
                case '2':
                    app_router.navigate('!pubGoods', {trigger: false});
            }
        }
    });

    var OrderList = Backbone.Collection.extend({
        url: apiUrl.addr + '?r=store.order.list',
        sync: function() {
            var self = this;
            $.ajax({
                url: this.url,
                type: 'get',
                data: {
                    status: orderStatus
                },
                success: function(data) {
                    if (data.success) {
                        self.reset(data.orders);
                        //显示页面
                        require.async('../nav', function(nav) {
                            nav.init(curStep);
                        });
                        $('#loading').fadeOut(200);
                        if (orderStatus == 3) {
                            if (data.orders.length == 0) {
                                $('.orders_no').hide();
                            }
                            else if (data.orders.length > 20) {
                                $('.orders_no').text("20+").show();
                            }
                            else {
                                $('.orders_no').text(data.orders.length).show();
                            }
                        }
                    } else {
                        self.goBackUrl();
                        $('#loading').fadeOut(200, function() {
                            alert(data.msg, function() {}, '提示', '确定');
                        });
                    }
                },
                error: function() {
                    self.goBackUrl();
                    $('#loading').fadeOut(200, function(e) {
                        alert('请检查当前网络', function() {}, '提示', '确定');
                    });
                }
            });
        },
        goBackUrl: function() {
            switch (curStep) {
                case '4':
                    app_router.navigate('!shopEidt', {trigger: false});
                    break;
                case '2':
                    app_router.navigate('!pubGoods', {trigger: false});
            }
        }
    });
       
    var MyWeidianView = Backbone.View.extend({
        el: '#myWeidian',

        model: new SkInfo(),

        collection: new OrderList(),

        initialize: function() {
            var self = this;
            //监听 collection reset 事件
            // this.collection.on('reset', function() {
            //     self.render(1);
            // });
            this.collection.on('reset', this.render, this);

            var width = ($(window).width() * 0.9 - 240)/3;
            $('#shareLayer li').css('margin-left', width);


            $('#shareLayer .cancel').on('touchstart', function() {
                $(this).addClass('taped');
            });

            $('#shareLayer .cancel').on('touchend', function() {
                $(this).removeClass('taped');
            });

            $('#shareLayer .cancel').on('click', function() {
                $('#shareLayer').fadeOut(400);
                $('#myWeidian').removeClass('blur_effect');
                $('nav').removeClass('blur_effect');
            });

            $('#shareLayer').on('touchstart', 'li', function(e) {
                var obj = $(e.target);
                obj.addClass('taped');
            });

            $('#shareLayer').on('touchend', 'li', function(e) {
                var obj = $(e.target);
                obj.removeClass('taped');
            });
        },

        m_template: function(data) {
            var temp = _.template($('#sk_info').html(), {model: data});
            return temp;
        },

        c_template: function(collection) {
            var temp = _.template($('#goods_list').html(), {collection: collection});
            return temp;
        },

        o_template: function(collection) {
            var temp = _.template($('#orders_list').html(), {collection: collection});
            return temp;
        },

        renderShop: function(data) {
            this.$('.tpl_part').html(this.m_template(data));
        },

        render: function(data) {
            if (orderStatus == 3) {


            }
            this.$('.orders_list').html(this.o_template(this.collection.toJSON()));
            // if (state == 0) {
                
            // } else if (state == 1) {
                
            // } else {
            //     this.$('.goods_list').html(this.c_template(data));
            //     this.$('.goods_no').text(data.length);
            // }
        },

        events: {
            'click .sk_info': 'showShopEdit',
            'click .first': 'listOrder',
            'click .second': 'listOrder',
            'click .third': 'listOrder',
            'click .last': 'listOrder',
            'click .last2': 'shareShop',
            'click .bottom_bar div': 'toggleEffect',
            'touchstart .last': 'tapBg',
            'touchend .last': 'notapBg',
            'click .order': 'toDetailView'
        },

        showShopEdit: function(e) {
            sessionStorage.skInfo = JSON.stringify(skInfo);
            var obj = $(e.currentTarget);
            app_router.navigate(obj.attr('rel'), {trigger: true});
        },

        toDetailView: function(e) {
            var obj = $(e.currentTarget);
            sessionStorage.order_id = obj.attr('data-id');
            app_router.navigate('!order_detail/' + sessionStorage.order_id, {trigger: true});
        },

        listOrder: function(e) {
            var obj = $(e.currentTarget);
            obj.parent().children().removeClass('actived');
            obj.addClass('actived');
            orderStatus = obj.data('status');
            this.collection.fetch();
        },

        shareShop: function() {
            this.$el.addClass('blur_effect');
            $('nav').addClass('blur_effect');
            $('#shareLayer').fadeIn(400);
        },

        tapBg: function() {
            $('.share').addClass('taped');
        },

        notapBg: function() {
            $('.share').removeClass('taped');
        },

        toggleEffect: function(e) {
            var obj = $(e.currentTarget);
            obj.addClass('taped').siblings().removeClass('taped');
        },

        taped: function(e) {
        	var obj = $(e.target);
        	obj.addClass('taped');
        },

        notap: function(e) {
        	var obj = $(e.target);
        	obj.removeClass('taped');
        }
    });

    var myWeidianView = new MyWeidianView();

    return {
        init: function(step) {
            if (step == '4' || step == '1') {
                curStep = step;
                $('#loading').fadeIn(200);
                myWeidianView.model.save();
            } else {
                if (step == '2') {
                    myWeidianView.render();
                }
                require.async('../nav', function(nav) {
                    nav.init(step);
                });
            }
        }
    };
});