define(function(require) {
    $('html, body').height('100%');

    var apiUrl = require('../apiConfig');

    var curStep=1;

    var fwP; //判断哪一页跳转到此页
    var isMod; //图片是否被修改过
    var orderInfo;
    var imgData; //用户上传的图像
    var order_id=0;

    var OrderDetail = Backbone.Model.extend({
        url: apiUrl.addr + '?r=store.order.detail',
        order_id: 0,
        sync: function() {
            var self = this;
            $.ajax({
                url: this.url,
                type: 'get',
                data: {
                    order_id: self.order_id
                },
                success: function(data) {
                    if (data.success) {
                        orderInfo = data.order;
                        orderDetailsView.render(data);
                        //显示页面
                        require.async('../nav', function(nav) {
                            nav.init(curStep);
                        });
                        $('#loading').fadeOut(200);
                    } else {
                        self.goBackUrl();
                        $('#loading').fadeOut(200, function() {
                            alert(data.msg, function() {}, '提示', '确定');
                        });
                    }
                },
                error: function() {
                    self.goBackUrl();
                    $('#loading').fadeOut(200, function(e) {
                        alert('请检查当前网络', function() {}, '提示', '确定');
                    });
                }
            });
        },
        goBackUrl: function() {
            switch (curStep) {
                case '1':
                    app_router.navigate('!myWeidian/1', {trigger: false});
            }
        }
    });

    var UpdateOrder = Backbone.Model.extend({
        url: apiUrl.addr + '?r=store.order.update',
        sync: function() {
            if (action == 2) {
                this.updatePrice();
            }
            else if (action == 3) {
                this.finishedOrder();
            }
            action = 0;
        },
        finishedOrder: function(){
            $.ajax({
                url: this.url,
                type: 'post',
                data: {
                    order_id: orderInfo.id,
                    tinyshop_id: sessionStorage.tinyshop_id,
                    action: "finished"
                },
                success: function(data) {
                    if (data.success) {
                        $('#loading').fadeOut(200, function() {
                            alert('保存成功', function() {
                                orderDetailsView.model.fetch();
                                //app_router.navigate('!pubGoods/3', {trigger: true});
                            }, '提示', '确定');
                        });
                    } else {
                        $('#loading').fadeOut(200, function() {
                            alert(data.msg, function() {}, '提示', '确定');
                        });
                    }
                },
                error: function() {
                    $('#loading').fadeOut(200, function(e) {
                        alert('请检查当前网络', function() {}, '提示', '确定');
                    });
                }
            });
        },

        updatePrice: function() {
            var self = this;
            var reg = /^(?:[1-9][0-9]*(?:\.[0-9]+)?|0(?:\.[0-9]+)?)$/;
            var txt = $('.order_price').val();
            if (txt == '') {
                alert('请输入订单价格', function() {}, '提示', '确定');
                $('#loading').fadeOut(200);
                return;
            } 
            if (!reg.test(txt)) {
                alert('订单价格格式错误，请检查', function() {}, '提示', '确定');
                $('#loading').fadeOut(200);
                return;
            } 

            $.ajax({
                url: this.url,
                type: 'post',
                data: {
                    order_id: orderInfo.id,
                    tinyshop_id: sessionStorage.tinyshop_id,
                    amount: txt,
                    action: 'update'
                },
                success: function(data) {
                    if (data.success) {
                        $('#loading').fadeOut(200, function() {
                            alert('保存成功', function() {
                                orderDetailsView.model.fetch();
                                orderDetailsView.order_cancel();
                            }, '提示', '确定');
                        });
                    } else {
                        $('#loading').fadeOut(200, function() {
                            alert(data.msg, function() {
                                orderDetailsView.order_cancel();
                            }, '提示', '确定');
                        });
                    }
                },
                error: function() {
                    $('#loading').fadeOut(200, function(e) {
                        alert('请检查当前网络', function() {
                            orderDetailsView.order_cancel();
                        }, '提示', '确定');
                    });
                }
            });
        }
    });
       
    var OrderDetailsView = Backbone.View.extend({
        el: '#order_detail',

        model: new OrderDetail(),

        updateModel: new UpdateOrder(),

        template: function(data) {
            var temp = _.template($('#order_detail_tpl').html(), {order: data.order, products: data.products});
            return temp;
        },

        render: function(data) {
            this.$(".content").html(this.template(data));
        },

        events: {
            'click .effect_icon': 'show_ul',
            'focus input': 'hide_ul',
            'focus textarea': 'hide_ul',
            'click .last': 'hide_ul',
            'click button.change_amount': 'change_amount',
            'click button.finish': 'order_finish',
            'click .cancel': 'order_cancel',
            'click .saveOrder': 'save_order',
            'touchstart li': 'taped',
            'touchend li': 'notap'
        },

        order_cancel: function(){
            this.$('.content').removeClass('blur_effect');
            $('nav').removeClass('blur_effect');
            $('#modifyOrderLayer').fadeOut(400);
        },

        change_amount: function(){
            this.$('.content').addClass('blur_effect');
            $('nav').addClass('blur_effect');
            $('.order_price').val(orderInfo.amount);
            $('#modifyOrderLayer').fadeIn(400);
        },

        save_order: function(){
            action = 2;
            $('#loading').fadeIn(200);
            this.updateModel.save();
        },

        order_finish: function(){
            action = 3;
            $('#loading').fadeIn(200);
            this.updateModel.save();
        },

        cDI: function() {
            $('#loading').fadeIn(200);
            this.updateModel.save();
        },

        show_ul: function() {
            this.$('ul').fadeIn(200);
        },

        hide_ul: function() {
            this.$('ul').hide();
        },

        taped: function(e) {
        	var obj = $(e.target);
        	obj.addClass('taped');
        },

        notap: function(e) {
        	var obj = $(e.target);
        	obj.removeClass('taped');
        }
    });

    var orderDetailsView = new OrderDetailsView();

    return {
        init: function(orderid) {
            imgData = '';
            var self = this;
            curStep = 1;
            $('#loading').fadeIn(200);
            orderDetailsView.model.order_id = orderid;
            orderDetailsView.model.fetch();

            require.async('../nav', function(nav) {
                nav.init(1);
            });
        }
    };
});