define(function(require, exports, module) {
    var config = require('../apiConfig');
    var utils = require('../utils');
    var defaultBuytype = "0000000002";
    var offset=0;
    var perpagenum=20;
    var firstperpage = 6;

    DianModel = require('../models/dian');
    GoodsCollection = require('../collections/goods');

    var DianView = Marionette.ItemView.extend({
        el: '#dian',
        template: '#dian-template',

        defaults: {
            model: null
        },

        cart: {
            op: 0,
            bp: 0,
            buytype: defaultBuytype,
        },
        
        collection: GoodsCollection,
        
        user: new UserModel(),

        initialize: function() {
            var self = this;
            this.listenTo(this.model, 'change', this.renderStore);
            this.listenTo(this.model, 'destroy', this.remove);
            this.listenTo(this.model, 'visible', this.toggleVisible);

            this.listenTo(this, 'cleanup', function(){
                this.cart = {
                    op: 0,
                    bp: 0,
                    buytype: defaultBuytype,
                };
                this.collection.reset();
            });

            this.listenTo(this, 'reset', function(){
                utils.showloading();
                self.collection.fetch({"perpage": firstperpage,"offset":0});
                self.model.fetch();
                self.render();
            });

            this.listenTo(this.model, 'error', function(data) {
                data = data || {};
                utils.hideloading();
                alert(data.msg || '请检查当前网络', function() {
                         app.router.back();
                }, '提示', '确定');
            });
            this.listenTo(this.collection, 'change', this.renderProductList);

            this.listenTo(this.collection, 'reset', this.renderProductList);
            this.listenTo(this.collection, 'reset', utils.hideloading);
        },

        events: {
            'click nav .backBtn': 'back',
//            'click li.product': 'addtoCartTap',
            'click span.numberadjest': 'numberAdjestTap',

            'focus .bargain_price': 'showBlurEffect',
            'blur .bargain_price': 'hideBlurEffect',
            'input .bargain_price': 'editPrice',
            'click .bargain_pricebtn':'showBlurEffect_btn',
            'click .settle': 'toSettlement',

            'touchstart .settle': 'taped',
            'touchend .settle': 'notap',
            'click .default_sort': 'defaultSort',
            'click .price_sort': 'priceSort',
            
            'click .trolley_icon': 'toTrolley', //显示购物车页面
            
            'tap #retailSale': 'retailSale',
            'tap #wholeSale': 'wholeSale',
            //'tap .buytype': 'buytype2',
            'click .payOrder': "buytype2",
            'click #getmore':"getmoreproduct"
        },
        
        getmoreproduct:function(e){
            offset += firstperpage;
            var obg = {"perpage": perpagenum,"offset":offset};
            this.collection.fetch(obg);
            this.$('#getmorepar').css('display','none');
        },

        back: function(e) {
            utils.hideloading();
            app.router.back();
        },

        onRender: function(){
            this.renderStore();
            this.renderProductList();
            this.renderCartInfo();
        },

        renderStore: function() {
            if (this.model && this.model.dian) {
                var tmpl = _.template($('#dian_info').html(), {model: this.model.dian, datanum: app.datanum});
                this.$('.dian-header').html(tmpl);
                this.$('.navTitle').html(this.model.dian.name);
                this.$('#canpifa').css('display','none');
            };
            if(this.cart.productnum>=10) {
               this.$('#canpifa').css('display','block');
            }
            else{
                 this.$('#canpifa').css('display','none');
            }
            console.log(this.collection);
        },

        renderProductList: function(){
            console.log("render Product list");

            var tmpl = _.template($('#goods_list').html(), {products: this.collection.toJSON()});
            this.$('.goods_list').html(tmpl);
            if( this.collection.productlength > firstperpage) {
                this.$('#getmorepar').css('display','block');
                if(this.collection.length>=this.collection.productlength){
                 this.$('#getmorepar').css('display','none');
                }
            }else{
                this.$('#getmorepar').css('display','none');
            }
            
        },

        renderCartInfo: function(resetPrice){
            if (typeof resetPrice === 'undefined') {
                resetPrice = true;
            }
            console.log("render Cart Info");
            var cart = this.cart;
            var opPrice = cart.op,
                bpPrice = cart.bp,
                totalPrice = 0;

            //如果成交价与订单价相同，或差额太大，则修改成交价
            var resetBpPrice = false;
            if( Math.abs(opPrice - bpPrice) < 0.001) {
                resetBpPrice = resetPrice &&  true;
            }
            if( bpPrice < 0.001) {
                resetBpPrice = true;
            }

            this.collection.each(function(p) {
                totalPrice += p.get('price') * p.get('amount');
            });
            opPrice = totalPrice;

            // if( Math.abs(opPrice - bpPrice) / opPrice > 0.5 ) {
            //     resetBpPrice = resetPrice && true;
            // }
            if(resetBpPrice) {
                bpPrice = opPrice;
            }

            this.cart.op = opPrice;
            this.cart.bp = bpPrice;

            var tmpl = _.template($('#cart_info').html(), {cart: this.cart});
            this.$('.cart_info').html(tmpl);

            if(this.cart.productnum>=10) {
               this.$('#canpifa').css('display','block');
            }
            else{
                 this.$('#canpifa').css('display','none');
            }
        },

        _findProduct: function( pid ) {
            //#return this.collection.find({id: pid});
            return _.find(this.collection.models, function(p){
                return (p.id + "") == (pid + ""); 
            });
        },

        // template : function(serialized_model) {
        //     var name = serialized_model.name;
        //     return _.template(this.templatename, {
        //         model2 : this.model,
        //         //some_custom_attribute : some_custom_key
        //     }, {variable: 'args'});
        // },

        addtocart: function(p, amount) {
            if(this.cartCheck() === false) {
                return false;
            }
            // var amount = amount || 0;
            var p1 = this._findProduct(p.id);
            if( ! p1 ) {
                var tmpp = $.extend({}, p, {amount: 1})
                this.collection.add(tmpp, {at: 0});
                p1 = this._findProduct(p.id);
            }
            else {
                if( amount<0&&p1.get('amount')<=0){
                    p1.set('amount', 0);   
                }else{
                    p1.set('amount', p1.get('amount') + amount);    
                }
                
            }

            if (p1.get('amount') <= 0) {
                // this.collection.remove(p1);
                this.renderProductList();
            }
            this.cart.productnum=this.chechprodunum();
            if(this.cart.productnum<10){
                  this.cart.buytype='0000000002';
                  this.$('.payOrder').checked=false;
            }

//            console.log(this.cart);
            this.renderCartInfo();
              
        },

        addtoCartTap: function(e) {
            var obj = $(e.currentTarget);
            console.log($(e.target));
            console.log(obj);
            if (e && e.target && $(e.target).hasClass('numberadjest')) {
                return;
            }

            var id = obj.attr('data-id'),
                goods_name = obj.children('.goods_name').text(),
                price = obj.children('.price').text(),
                image_src = obj.attr('data-image');
            this.addtocart({id:id, name:goods_name, price:price, image_src:image_src}, 1);
        },

        numberAdjestTap: function(e){
            event.preventDefault();
            event.stopPropagation();
            var obj = $(e.currentTarget);
            var id = obj.parents('li').attr('data-id');
            var m = 0;
            if (obj.hasClass('decrease')) {
                m = -1;
            }
            else if( obj.hasClass('increase')) {
                m = 1;
            }
        
            this.addtocart({id:id}, m);
            
        },
        
        chechprodunum:function(){
            var productAmount = 0;//商品数量
            this.collection.each(function(p) {
                productAmount += p.get('amount');
            });
            return productAmount;
        },

        showBlurEffect: function() {
            this.$('.tpl_part, .sort_column, .goods_list').addClass('blur');
        },
        showBlurEffect_btn:function(){
            this.$('.tpl_part, .sort_column, .goods_list').addClass('blur');
            this.$('.bargain_price').focus();
        },

        hideBlurEffect: function(e) {
            this.$('.tpl_part, .sort_column, .goods_list').removeClass('blur');
            this.saveBarginPrice(e);
        },

        saveBarginPrice: function(e){
            var obj = $(e.target);
              var raw_price = this.cart.bp;
              var min_price = (parseFloat(raw_price) * 0.5);
              var max_price = (parseFloat(raw_price) * 1.5);
              if(min_price<=0){
                  min_price=0;
              }
              if (( parseFloat(obj.val()) < min_price) || (obj.val() > max_price))  {
                obj.val((this.cart.bp).toFixed(2));
                alert( ' 议价金额必须在 ' + min_price.toFixed(2)  + ' - ' + max_price.toFixed(2) + ' 之间');
              } else {
                this.cart.bp = parseFloat(obj.val() * 1);
                this.renderCartInfo(true);   
              }
                if(this.cart.productnum>=10){
                     this.$('#canpifa').css('display','block');
                }else{
                     this.$('#canpifa').css('display','none');
                }
        },

        editPrice: function(e) {
            var obj = $(e.target);
            if (obj.val().length > 8) {
                obj.val(obj.val().slice(0, -1));
            }
        },

        cartCheck: function(e) {
            if (this.cart.op >= 9000000) {
                alert('总价太高了，请重新选购', function() {}, '提示', '确定');
                return false;
            }
            return true;
        },


        toSettlement: function() {
            var productAmount = 0;//商品数量
            var obj = this.$('.settle');
            var reg = /^(?:[1-9][0-9]*(?:\.[0-9]+)?|0(?:\.[0-9]+)?)$/;
            
          //  console.log(this.collection);
            this.collection.each(function(p) {
              //  console.log(p.attributes.amount);
                productAmount+=p.attributes.amount;
            });

            var goContinue = false;
        
            if( Math.abs(this.cart.op - this.cart.bp) / this.cart.op > 0.5 ) {
                alert('成交价不能小于原订单金额', function() {}, '提示', '确定');
                return;
            }

            if (this.cart.bp == 0) {
                alert('成交价不能为0', function() {}, '提示', '确定');
            }
            else if (this.cart.bp < 10) {
                alert('成交价不能小于十元', function() {}, '提示', '确定');
            }
            else if (this.cart.bp > 10000000) {
                alert('成交价不能大于一千万元', function() {}, '提示', '确定');
            }
            else if (this.collection.length == 0) {
                alert('请至少选择一件商品', function() {}, '提示', '确定');
            }
            else if ( this.cart.buytype == null ) {
                alert('交易方式必须选择，请选择零售或批发', function() {}, '提示', '确定');
            }
            else if (productAmount < 10 && this.cart.buytype == "0000000000") {
                navigator.notification.confirm('购买10件商品以上才可享受批发优惠', function(buttonIndex){
                    if (buttonIndex == 0) {
                        goContinue = true;
                    }
                }, '提示', '以零售方式继续交易,重新选购');
            }
            else {
                goContinue = true;
            }

            if(goContinue) {
                sessionStorage.amount = this.cart.bp.toFixed(2);
                sessionStorage.products = JSON.stringify(this.collection);
                sessionStorage.buytype = this.cart.buytype;
                app.router.navigate(this.$('.settle').attr('rel'), {trigger: true});
            }
        },

        t_template: function() {
            //var temp = _.template($('#trolley_layer_tpl').html(), {products: this.products});
            //return temp;
        },

        buytype2: function(e) {
            var target = $(e.currentTarget);
            if( target.prop("checked") ) {
                this.cart.buytype = target.val();
            }
            else {
                this.cart.buytype = defaultBuytype;
            }
        },

        retailSale: function(e){
            this.$('#retailSale').addClass('taped')
            this.$('#wholeSale').removeClass('taped');
        },

        wholeSale: function(e){
            this.$('#wholeSale').addClass('taped')
            this.$('#retailSale').removeClass('taped');
        },

        defaultSort: function() {
            this.$('.default_sort').addClass('taped').toggleClass('toLow');
            this.$('.price_sort').removeClass('taped');

            var models = _.sortBy(this.collection.models, function(p){
                return parseFloat(p.get('id')); 
            });
            models.reverse();
            this.collection.reset(models);
        },

        priceSort: function() {
            this.$('.price_sort').addClass('taped').toggleClass('toLow');
            this.$('.default_sort').removeClass('taped');

            var models = _.sortBy(this.collection.models, function(p){
                return parseFloat(p.get('price')); 
            });
            if( this.$('.price_sort').hasClass('toLow') ) {
                models.reverse();
            }
            this.collection.reset(models);
        },

        toTrolley: function() {
            $('#trolley_layer').height($('#enterWeidian').height());
            this.render(3);
            $('#trolley_layer').fadeIn(300);
            $(window).scrollTop(0);
            $('nav .leftButton').css('visibility', 'hidden'); // 隐藏导航栏返回按钮
        },


        taped: function(e) {
            var obj = $(e.target);
            obj.addClass('taped');
        },

        notap: function(e) {
            var obj = $(e.target);
            obj.removeClass('taped');
        }
    });

    var _view = null;
    
    // 对外提供 foo 属性
    //exports.View = HomepageView;
    module.exports = {
        getView: function() {
            if (_view == null) {
                _view = new DianView({
                    model: DianModel,
                    collection: GoodsCollection,
                });
                _view.render();
            }
            return _view;
            
        }
    }

});
